using System.Net.Mime;
using System.Text;
using Asp.Versioning;
using JacobTechEncryption;
using JacobTechEncryption.Enums;
using LuskiServer.Classes;
using LuskiServer.Classes.TableDef;
using LuskiServer.Classes.v1.OutGoing;
using LuskiServer.Classes.WebTypes;
using LuskiServer.Enums;
using Microsoft.AspNetCore.Mvc;

namespace LuskiServer.Controllers.v1;

/// <summary>
/// This controller is for handing all request relating to encryption keys.
/// </summary>
[ApiVersion(1)]
[ApiController]
public class KeysController : ControllerBase
{
    /// <summary>
    /// Return a public RSA key to encrypt information directed for the server.
    /// </summary>
    /// <returns></returns>
    [HttpGet]
    [ProducesResponseType(StatusCodes.Status200OK)]
    [Produces(MediaTypeNames.Application.Xml)]
    [Route(Luski.Info.Routes.Default.Base + "/PublicKey")]
    public IActionResult PublicKey()
    {
        return File(Encoding.UTF8.GetBytes(Luski.Encryption.Keys.PublicKey), "application/xml");
    }

    /// <summary>
    /// Make a delete request to this endpoint to delete a key from the server.
    /// </summary>
    /// <param name="token">Your Luski token for the server.</param>
    /// <param name="keyid">The public key id you want to delete on the server.</param>
    /// <returns></returns>
    [HttpDelete]
    [Produces(MediaTypeNames.Application.Json)]
    [ProducesResponseType(StatusCodes.Status202Accepted)]
    [ProducesResponseType(typeof(HTTPResponse), StatusCodes.Status403Forbidden)]
    [Route(Luski.Info.Routes.Default.Base)]
    public IActionResult Delete([FromHeader(Name = "id")]long keyid, [FromHeader(Name = "token")]string? token)
    {
        try
        {
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
            if (!Tables.Keys.TryRead(Keys.Owner, out long Owner, Keys.ID.CreateParameter(keyid)) || (Owner != ID || !Luski.HasPermissions(ID, ServerPermissions.DeleteKeys)))
                return this.ResponseCodeToResult(ErrorCode.Forbidden);

            Tables.Keys.DeleteRow(Keys.ID.CreateParameter(keyid));
            return StatusCode(202);
        }
        catch (Exception ex)
        {
            return this.ShowError(ex);
        }
    }
    
    /// <summary>
    /// Make a post request to this URL to add a new key to the server.
    /// </summary>
    /// <param name="token">Your Luski token for the server</param>
    /// <param name="keyreq">The public key info you want to store on the server</param>
    /// <returns></returns>
    [HttpPost]
    [Consumes(typeof(ClientKeyPostReqest), MediaTypeNames.Application.Json)]
    [Produces(MediaTypeNames.Application.Json)]
    [ProducesResponseType(StatusCodes.Status202Accepted)]
    [ProducesResponseType(typeof(HTTPResponse), StatusCodes.Status403Forbidden)]
    [Route(Luski.Info.Routes.Default.Base)]
    public IActionResult Post([FromBody]ClientKeyPostReqest keyreq, [FromHeader(Name = "token")]string? token)
    {
        try
        {
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;

            Luski.Snowflake sf = Luski.Snowflake.GenerateSnowflake(Luski.Config.ServerEpoch);
            Tables.Keys.Insert(
                Keys.ID.CreateParameter(sf.ID),
                Keys.Owner.CreateParameter(ID),
                Keys.EncryptionType.CreateParameter(keyreq.encryption_type),
                Keys.KeyData.CreateParameter(Convert.FromBase64String(keyreq.key_data)));
            return StatusCode(202);
        }
        catch (Exception ex)
        {
            return this.ShowError(ex);
        }
    }
    
    /// <summary>
    /// Make a get request to this endpont to get a public key to a user.
    /// </summary>
    /// <param name="userid">The user id you want to get a key from.</param>
    [HttpGet]
    [Produces(MediaTypeNames.Application.Json)]
    [ProducesResponseType(typeof(HTTPResponse), StatusCodes.Status403Forbidden)]
    [Route(Luski.Info.Routes.Default.Base + "/UserKeys")]
    public IActionResult GetUserKeys([FromRoute(Name = "id")]long userid)
    {
        try
        {
            if (!Tables.Users.TryRead(Users.ID, out _, Users.ID.CreateParameter(userid))) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            bool UserOnline = Tables.Sessions.TryReadRows(out SessionsRow[] usersessions, Sessions.User.CreateParameter(userid));
            List<UserKeyGetRequest> req = new();
            if (!UserOnline)
            {
                foreach (StorageRow StorageRow in Tables.Storage.ReadRows(uint.MaxValue, Storage.AccountID.CreateParameter(userid)))
                {
                    KeyRow Row = Tables.Keys.ReadRow(Keys.ID.CreateParameter(StorageRow.OffileKey));
                    req.Add(new UserKeyGetRequest()
                    {
                        id = Row.ID,
                        owner = Row.Owner,
                        encryption_type = Row.EncryptionType,
                        key_data = Convert.ToBase64String(Row.KeyData)
                    });
                }
                
            }
            else
            {
                List<long> b = new();
                foreach (SessionsRow sess in usersessions)
                {
                    if (!b.Contains(sess.StorageID)) b.Add(sess.StorageID);
                    KeyRow Row = Tables.Keys.ReadRow(Keys.ID.CreateParameter(sess.SessionKey));
                    req.Add(new UserKeyGetRequest()
                    {
                        id = Row.ID,
                        owner = Row.Owner,
                        encryption_type = Row.EncryptionType,
                        key_data = Convert.ToBase64String(Row.KeyData)
                    });
                }
                
                foreach (StorageRow StorageRow in Tables.Storage.ReadRows(uint.MaxValue, Storage.AccountID.CreateParameter(userid)))
                {
                    if (b.Contains(StorageRow.ID)) continue;
                    KeyRow Row = Tables.Keys.ReadRow(Keys.ID.CreateParameter(StorageRow.OffileKey));
                    req.Add(new UserKeyGetRequest()
                    {
                        id = Row.ID,
                        owner = Row.Owner,
                        encryption_type = Row.EncryptionType,
                        key_data = Convert.ToBase64String(Row.KeyData)
                    });
                }
            }
            return this.ResponseToResult(new UserKeysGetRequest()
            {
                keys = req.ToArray()
            });
        }
        catch (Exception ex)
        {
            return this.ShowError(ex);
        }
    }
    
    /// <summary>
    /// Make a get request to this endpont to get a public key from the server.
    /// </summary>
    /// <param name="keyid">The public key id you want to get from the server.</param>
    [HttpGet]
    [Produces(MediaTypeNames.Application.Json)]
    [ProducesResponseType(typeof(HTTPResponse), StatusCodes.Status403Forbidden)]
    [Route(Luski.Info.Routes.Default.Base)]
    public IActionResult Get([FromHeader(Name = "id")]long keyid)
    {
        try
        {
            if (!Tables.Keys.TryReadRow(out KeyRow Row, Keys.ID.CreateParameter(keyid))) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            return this.ResponseToResult(new UserKeyGetRequest()
            {
                id = Row.ID,
                owner = Row.Owner,
                encryption_type = Row.EncryptionType,
                key_data = Convert.ToBase64String(Row.KeyData)
            });
        }
        catch (Exception ex)
        {
            return this.ShowError(ex);
        }
    }
    
    /// <summary>
    /// Make a post request to this URL to set you offline data key.
    /// </summary>
    /// <param name="token">Your Luski token for the server.</param>
    /// <param name="keyreq">The key you want to set for when you go offline.</param>
    /// <returns></returns>
    [HttpPost]
    [Consumes(typeof(ClientKeyPostReqest), MediaTypeNames.Application.Json)]
    [Produces(MediaTypeNames.Application.Json)]
    [ProducesResponseType(StatusCodes.Status202Accepted)]
    [ProducesResponseType(typeof(HTTPResponse), StatusCodes.Status403Forbidden)]
    [ProducesResponseType(typeof(HTTPResponse), 500)]
    [Route(Luski.Info.Routes.Default.Base + "/SetOfflineKey")]
    public IActionResult SetOfflineKey([FromBody]ClientKeyPostReqest keyreq, [FromHeader(Name = "token")]string? token, [FromHeader(Name = "storage_id")]long StorageID)
    {
        try
        {
            string key = "";
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
            byte[][]? data = Tables.Storage.Read(Storage.Data, Storage.ID.CreateParameter(ID));
            if (data is not null && data.Any()) return this.ShowError(ErrorCode.Forbidden, "you cant change your key until you download your data");
            long k = 0;
            if ((k = Tables.Storage.Read(Storage.OffileKey, Storage.ID.CreateParameter(ID))) != 0)
            {
                Tables.Keys.DeleteRow(Keys.ID.CreateParameter(k));
            }
            Luski.Snowflake sf = Luski.Snowflake.GenerateSnowflake(Luski.Config.ServerEpoch);
            Tables.Keys.Insert(
                Keys.ID.CreateParameter(sf.ID),
                Keys.Owner.CreateParameter(ID),
                Keys.EncryptionType.CreateParameter(keyreq.encryption_type),
                Keys.KeyData.CreateParameter(Convert.FromBase64String(keyreq.key_data)));
            Tables.Storage.Update(Storage.ID, ID,
                Storage.OffileKey.CreateParameter(sf.ID));
            return StatusCode(202);
        }
        catch (Exception ex)
        {
            return this.ShowError(ex);
        }
    }
}