using Asp.Versioning;
using LuskiServer.Classes;
using LuskiServer.Classes.TableDef;
using LuskiServer.Classes.v1.OutGoing;
using LuskiServer.Classes.WebTypes;
using LuskiServer.Enums;
using Microsoft.AspNetCore.Mvc;
using Microsoft.CodeAnalysis;

namespace LuskiServer.Controllers.v1;

[ApiVersion(1)]
[ApiController]
public class SocketCategoryController : ControllerBase
{
    [HttpPost]
    [Route(Luski.Info.Routes.Default.Base)]
    public IActionResult Post([FromBody]UserChannelRequest ChanReq)
    {
        try
        {
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
            if (!Luski.HasAccessToCategory(ID, ChanReq.parent, ServerPermissions.CreateChannels)) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            DateTime epoch = Luski.Config.ServerEpoch;
            List<long> RoleOverides = new(), UserOverides = new();
            if (ChanReq.role_overides is not null && ChanReq.role_overides.Any())
            {
                foreach (UserRoleOverideRequest over in ChanReq.role_overides)
                {
                    Luski.Snowflake overSnowflake = Luski.Snowflake.GenerateSnowflake(epoch);
                    Tables.ServerRoleOverides.Insert(
                        ServerRoleOverides.ID.CreateParameter(overSnowflake.ID),
                        ServerRoleOverides.RoleID.CreateParameter(over.role_id),
                        ServerRoleOverides.Overides.CreateParameter(over.overides));
                    RoleOverides.Add(overSnowflake.ID);
                }
            }
            if (ChanReq.user_overides is not null && ChanReq.user_overides.Any())
            {
                foreach (UserPermOverideRequest over in ChanReq.user_overides)
                {
                    Luski.Snowflake overSnowflake = Luski.Snowflake.GenerateSnowflake(epoch);
                    Tables.UserRoleOverides.Insert(
                        UserRoleOverides.ID.CreateParameter(overSnowflake.ID),
                        UserRoleOverides.UserID.CreateParameter(over.user_id),
                        UserRoleOverides.Overides.CreateParameter(over.overides));
                    UserOverides.Add(overSnowflake.ID);
                }
            }

            epoch = DateTime.UtcNow.Date;
            Luski.Snowflake chanSnowflake = Luski.Snowflake.GenerateSnowflake(epoch);
            Tables.Channels.Insert(
                Channels.ID.CreateParameter(chanSnowflake.ID),
                Channels.Parent.CreateParameter(ChanReq.parent),
                Channels.Type.CreateParameter(ChanReq.type),
                Channels.Epoch.CreateParameter(epoch.Date),
                Channels.Name.CreateParameter(Convert.FromBase64String(ChanReq.name)),
                Channels.Description.CreateParameter(Convert.FromBase64String(ChanReq.description)),
                Channels.RoleOverides.CreateParameter(RoleOverides.ToArray()),
                Channels.UserOverides.CreateParameter(UserOverides.ToArray()),
                Channels.TitleEncryptionKey.CreateParameter(ChanReq.title_encryption_key),
                Channels.DescriptionEncryptionKey.CreateParameter(ChanReq.description_encryption_key),
                Channels.EncryptionKeys.CreateParameter(ChanReq.encryption_keys),
                Channels.TitleEncoderType.CreateParameter(ChanReq.title_encoder_type),
                Channels.DescriptionEncoderType.CreateParameter(ChanReq.description_encoder_type),
                Channels.EncoderTypes.CreateParameter(ChanReq.encoder_types));
            return this.ResponseToResult(new UserChannelRequest(){ id = ID});
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
    
    [HttpDelete]
    [Route(Luski.Info.Routes.Default.Base)]
    public IActionResult Delete([FromHeader(Name = "id")]long Channel)
    {
        try
        {
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
            if (!Luski.HasAccessToCategory(ID, Channel, ServerPermissions.DeleteChannels)) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            Tables.Channels.DeleteRow(Channels.ID.CreateParameter(Channel));
            Tables.Messages.DeleteRow(Messages.ChannelID.CreateParameter(Channel));
            return StatusCode(StatusCodes.Status202Accepted);
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
    
    [HttpPatch]
    [Route(Luski.Info.Routes.Default.Base)]
    public IActionResult Patch([FromHeader(Name = "id")]long Channel)  //TODO Add Channel edit latter
    {
        return null!;
    }
    
    [HttpGet]
    [Route(Luski.Info.Routes.Default.Base)]
    public IActionResult Get([FromHeader(Name = "id")]long Channel)
    {
        try
        {
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
            if (!Luski.HasAccessToCategory(ID, Channel, out IReadOnlyList<ServerPermissions> opt, ServerPermissions.ViewCategories, ServerPermissions.ViewChannels)) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            var chan = Tables.Categories.ReadRow(Categories.ID.CreateParameter(Channel));
            if (opt.Contains(ServerPermissions.ViewCategories))
            {
                chan.InnerCategories = Array.Empty<long>();
            }
            if (opt.Contains(ServerPermissions.ViewChannels))
            {
                chan.Channels = Array.Empty<long>();
            }
            return this.ResponseToResult(new UserCategoryResponse()
            {
                name = Convert.ToBase64String(chan.Name),
                description = Convert.ToBase64String(chan.Description),
                id = Channel,
                channels = chan.Channels,
                inner_categories = chan.InnerCategories,
                parent = chan.Parent,
                title_encryption_key = chan.TitleEncryptionKey,
                description_encryption_key = chan.DescriptionEncryptionKey,
                title_encoder_type = chan.TitleEncoderType,
                description_encoder_type = chan.DescriptionEncoderType,
                member_overides = chan.UserOverides,
                role_overides = chan.RoleOverides,
                color = chan.Color
            });
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
}